# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"DEFAULTCOLS": "database2.ipynb",
         "PandasDataFrameAttribute": "database2.ipynb",
         "Database": "database2.ipynb",
         "updateWithDfs": "database2.ipynb",
         "lambdaPresignUpload": "database2.ipynb",
         "lambdaIngestUpload": "database2.ipynb",
         "lambdaSingleBranchQuery": "database2.ipynb",
         "lambdaAllBranchesQuery": "database2.ipynb",
         "DatabaseHelper": "helper.ipynb",
         "Querier": "query.ipynb",
         "DatabaseS3": "s3.ipynb",
         "Updater": "update.ipynb"}

modules = ["database2.py",
           "helper.py",
           "query.py",
           "s3.py",
           "update.py"]

doc_url = "https://thanakijwanavit.github.io/villaPriceDatabase/"

git_url = "https://github.com/thanakijwanavit/villaPriceDatabase/tree/master/"

def custom_doc_links(name): return None
